from multiprocessing.dummy import current_process
from operator import ne
from dsim.node import Node
from typing import List, TypeVar, Generic, Union

T = TypeVar('T')


class Larray:

    def __init__(self, head: Node | None = None) -> None:
        self._head = head
        if head is None:
            self._length = 0
        else:
            self._length = self._get_length()

    def _head_is_none(self) -> bool:
        return self._head is None

    def _get_length(self) -> int:

        if self._head_is_none():
            return 1

        length = 1
        cur_node = self._head

        while True:

            if cur_node.next is None:
                break
            else:
                length += 1
                cur_node = cur_node.next

        return length

    @property
    def head(self) -> Node:
        return self._head

    @head.setter
    def head(self, new_head: Node) -> None:
        self._head = new_head
        self._length = self._get_length()

    @property
    def length(self) -> int:
        return self._length

    def __str__(self) -> str:
        if self._head_is_none():
            return "< *>"

        string = ""
        cur_node = self._head
        while True:
            if string == "":
                string += f' *{str(cur_node)}'
            else:
                string += f', {str(cur_node)}'

            if cur_node.next is None:
                break
            else:
                cur_node = cur_node.next

        return '<' + string + '>'

    def append(self, new_node: Node) -> None:
        if self._head_is_none():
            self._head = new_node
            self._length = self._get_length()
            return

        cur_node = self._head
        while True:
            if cur_node.next is None:
                cur_node.next = new_node
                self._length = self._get_length()
                return
            else:
                cur_node = cur_node.next

    def list(self) -> List[T]:
        if self._head_is_none():
            return []

        cur_node = self._head
        new_list = [cur_node.value]

        while True:
            if cur_node.next is None:
                return new_list
            else:
                cur_node = cur_node.next
                new_list.append(cur_node.value)

    def remove_end(self) -> None:
        if self._head_is_none():
            return

        if self._head.next is None:
            self._head = None
            self._length = 0
            return

        cur_node = self._head

        while True:
            if cur_node.next.next is None:
                cur_node.next = None
                self._length = self._get_length()
                return
            else:
                cur_node = cur_node.next

    def remove_start(self) -> None:
        if self._head_is_none():
            return

        self._head = self._head.next
        self._length = self._get_length()
