from typing import Generic, TypeVar

T = TypeVar('T')


class dsimNodeException(Exception):
    def __init__(self, *args: object) -> None:
        super().__init__(*args)


class Node(Generic[T]):

    def __init__(self, value: T = None) -> None:
        if value is None:
            raise dsimNodeException(
                f'Node value cannot be set as {type(None)}')
        self._value = value
        self._next = None

    @property
    def value(self) -> T:
        return self._value

    @property
    def next(self) -> "Node[T]":
        return self._next

    @next.setter
    def next(self, new_node: "Node[T]") -> None:
        self._next = new_node

    def _invalid_argument_type_exception(self, operator_type: str, other_node: any) -> any:
        raise dsimNodeException(
            f'{type(self)} cannot be compared with ({operator_type}) to the argument {type(other_node)}')

    def __str__(self) -> str:
        return f'{self._value}'

    def __eq__(self, other_node: "Node[T]") -> bool:
        if isinstance(other_node, Node):
            return self._value == other_node.value
        return self._invalid_argument_type_exception('==', other_node)

    def __ne__(self, other_node: "Node[T]") -> bool:
        return not self.__eq__(other_node)

    def __gt__(self, other_node: "Node[T]") -> bool:
        if isinstance(other_node, Node):
            return self._value > other_node.value
        return self._invalid_argument_type_exception(">", other_node)

    def __lt__(self, other_node: "Node[T]") -> bool:
        if isinstance(other_node, Node):
            return self._value < other_node.value
        return self._invalid_argument_type_exception("<", other_node)

    def __ge__(self, other_node: "Node[T]") -> bool:
        return self.__eq__(other_node) or self.__gt__(other_node)

    def __le__(self, other_node: "Node[T]") -> bool:
        return self.__eq__(other_node) or self.__lt__(other_node)
